/***********************************************************************
 *                    libosa - OS and Stream Agent                     *
 *                Copyright (C) 2001 Michael Brownlow                  *
 *                                                                     *
 * This program is free software; you can redistribute it and/or modify*
 * it under the terms of the GNU General Public License as published by*
 * the Free Software Foundation; either version 2 of the License, or   *
 * (at your option) any later version.                                 *
 *                                                                     *
 * This program is distributed in the hope that it will be useful,     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 * GNU General Public License for more details.                        *
 *                                                                     *
 * You should have received a copy of the GNU General Public License   *
 * along with this program; if not, write to the Free Software         *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
 *                                                                     *
 * For questions and comments, please email the author at:             *
 * mike@wsmake.org                                                     *
 ***********************************************************************/
#ifndef __OSA_H__
#define __OSA_H__ 1

#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <stdarg.h>

#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
# define __BEGIN_DECLS extern "C" {
# define __END_DECLS }
#else
# define __BEGIN_DECLS /* empty */
# define __END_DECLS /* empty */
#endif

#if !defined(__P)
# if defined(__cplusplus)
#  if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 7)
#   define __P(args)          args throw ()
#  else
#   define __P(args)          args
#  endif
# elif defined __STDC__ && __STDC__ > 0
#  define __P(args)           args
# else
#  define __P(args)           ()
# endif
#endif

#if !defined(__PMT)
# if defined(__cplusplus)
#  define __PMT(args)         args
# elif defined __STDC__ && __STDC__ > 0
#  define __PMT(args)         args
# else
#  define __PMT(args)         ()
# endif
#endif

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

__BEGIN_DECLS

typedef struct _osaFileList_t {
  char *filename;
  int directory;
  struct _osaFileList_t *next;
} osaFileList_t;

/* Memory Management */
void    *osaMalloc  __P((size_t sz));
void    *osaRealloc __P((void *mem, size_t sz));
void    *osaFree    __P((void *mem));

/* File I/O, Manipulation */
int      osaOpen          __P((const char *filename, int flags, mode_t mode));
int      osaClose         __P((int fd));
FILE    *osaFileOpen      __P((const char *filename, const char *mode));
FILE    *osaFDOpen        __P((int fd, const char *mode));
int      osaFileClose     __P((FILE *file));
int	 osaFileRemove	  __P((const char *filename));
int      osaFileCopyMode  __P((const char *from_file, const char *to_file));
int      osaFileCopyTimes __P((const char *from_file, const char *to_file));
int      osaFileExists    __P((const char *filename));
time_t   osaFileLastMod   __P((const char *filename));


/* Verbosity and Error handling */
void     osaSetPrintLevel __P((char level));
void     osaPrintError    __P((char level, const char *topic, ...));
void     osaVPrintError   __P((char level, const char *fmt, va_list ap));
void     osaPrintf        __P((char level, const char *str, ...));

/* Directory Manipulation */
char            *osaDirCurrent  __P((char *buffer));
int              osaDirCreate   __P((const char *directory, mode_t mode));
int              osaDirExists   __P((const char *filename));
char            *osaDirCollapse __P((const char *, const char *));
osaFileList_t   *osaDirList     __P((const char *directory));
osaFileList_t   *osaDirListFree __P((osaFileList_t *fl));

/* String Management */
char    *osaStrDup __P((const char *str));
char    *osaStrCat __P((char *str1, const char *str2));
int      osaStrCmp __P((const char *str1, const char *str2));
char    *osaStrStr __P((const char *haystack, const char *needle));
size_t   osaStrLen __P((const char *str));

__END_DECLS

#endif /* __OSA_H__ */
