/***********************************************************************
 *                    libosa - OS and Stream Agent                     *
 *                Copyright (C) 2001 Michael Brownlow                  *
 *                                                                     *
 * This program is free software; you can redistribute it and/or modify*
 * it under the terms of the GNU General Public License as published by*
 * the Free Software Foundation; either version 2 of the License, or   *
 * (at your option) any later version.                                 *
 *                                                                     *
 * This program is distributed in the hope that it will be useful,     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 * GNU General Public License for more details.                        *
 *                                                                     *
 * You should have received a copy of the GNU General Public License   *
 * along with this program; if not, write to the Free Software         *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
 *                                                                     *
 * For questions and comments, please email the author at:             *
 * mike@wsmake.org                                                     *
 ***********************************************************************/
#include "osa.h"

#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>
#include <dirent.h>
#include <stdlib.h>
#include <string.h>

char *osaDirCurrent(char *buffer)
{
  static size_t sz = 512;
  
  /* FIXME: add testing of getcwd return value and size accordingly */
  buffer = (char *)osaMalloc(sz);
  return getcwd(buffer, sz);
}

int osaDirCreate(const char *directory, mode_t mode)
{
  /* FIXME: port parent dir test code from original */
  if(mkdir(directory, mode) == -1) {
    return errno;
  } else {
    return 0;
  }
}

int osaDirExists(const char *filename)
{
  struct stat statbuf;
  mode_t modes;
  int err;

  err = stat(filename, &statbuf);
  if(!err) {
    modes = statbuf.st_mode;
    if(S_ISDIR(modes))
      return 1;
    else
      return 0;
  }

  return 0;
}

osaFileList_t *osaDirList(const char *directory)
{
  DIR *dp;
  struct dirent *entry;
  osaFileList_t *fl = NULL, *flt = NULL;
  char *testdir = NULL;
  size_t dlen = strlen(directory), elen;

  if((dp = opendir(directory)) == NULL) {
    osaPrintError(1, "Couldn't open directory `%s'", directory);
    return NULL;
  }

  while((entry = readdir(dp)) != NULL) {
    if(fl == NULL) {
      fl = (osaFileList_t *)osaMalloc(sizeof(osaFileList_t));
      flt = fl;
    } else {
      flt->next = (osaFileList_t *)osaMalloc(sizeof(osaFileList_t));
      flt = flt->next;
    }
    flt->next = NULL;
    flt->filename = osaStrDup(entry->d_name);
    elen = strlen(entry->d_name);
    testdir = osaMalloc(dlen+elen+2);
    memset(testdir,0,dlen+elen+2);
    testdir = memcpy(testdir, directory, dlen);
    memcpy(testdir+(dlen*sizeof(char)), "/", 1);
    memcpy(testdir+((dlen+1)*sizeof(char)), entry->d_name, elen);
    if(osaDirExists(testdir)) {
      flt->directory = 1;
    } else {
      flt->directory = 0;
    }
    free(testdir);
  }

  return fl;
}

osaFileList_t *osaDirListFree(osaFileList_t *fl)
{
  osaFileList_t *flt = fl;

  while(flt != NULL) {
    osaFree(flt->filename);
    fl = flt;
    flt = flt->next;
    osaFree(fl);
  }

  return NULL;
}
