/***********************************************************************
 *                    libosa - OS and Stream Agent                     *
 *                Copyright (C) 2001 Michael Brownlow                  *
 *                                                                     *
 * This program is free software; you can redistribute it and/or modify*
 * it under the terms of the GNU General Public License as published by*
 * the Free Software Foundation; either version 2 of the License, or   *
 * (at your option) any later version.                                 *
 *                                                                     *
 * This program is distributed in the hope that it will be useful,     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 * GNU General Public License for more details.                        *
 *                                                                     *
 * You should have received a copy of the GNU General Public License   *
 * along with this program; if not, write to the Free Software         *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
 *                                                                     *
 * For questions and comments, please email the author at:             *
 * mike@wsmake.org                                                     *
 ***********************************************************************/
#include "osa.h"

#include <stdio.h>
#include <errno.h>
#include <utime.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

extern int osaErrno;

int osaOpen(const char *filename, int flags, mode_t mode)
{
  int fd;

  if((fd = open(filename, flags, mode)) == -1) {
    osaErrno = errno;
  }

  return fd;
}

int osaClose(int fd)
{
  int ret;

  if((ret=close(fd)) != 0) {
    osaErrno = errno;
  }

  return ret;
}

FILE *osaFileOpen(const char *filename, const char *mode)
{
  FILE *f;

  if((f = fopen(filename, mode)) == NULL) {
    osaErrno = errno;
    return NULL;
  }

  return f;
}

FILE *osaFDOpen(int fd, const char *mode)
{
  FILE *f;
  
  if((f = fdopen(fd, mode)) == NULL) {
    osaErrno = errno;
    return NULL;
  }

  return f;
}

int osaFileClose(FILE *file)
{
  if(fclose(file) == EOF) {
    osaErrno = errno;
    return EOF;
  }

  return 0;
}

int osaFileRemove(const char *filename)
{
  if(unlink(filename) == -1) {
    osaErrno = errno;
    return -1;
  }

  return 0;
}

int osaFileCopyMode(const char *from_file, const char *to_file)
{
  struct stat statbuf;
  int err;

  err = stat(from_file, &statbuf);
  if(err == -1) {
    osaErrno = errno;
    if(osaErrno == ENOENT)
      return osaErrno;
  }

  err = chmod(to_file, statbuf.st_mode);

  if(err == -1) {
    osaErrno = errno;
    return osaErrno;
  }

  return 0;
}

int osaFileCopyTimes(const char *from_file, const char *to_file)
{
  struct stat statbuf;
  struct utimbuf utimebuff;
  int err;

  err = stat(from_file, &statbuf);
  if(err == -1) {
    osaErrno = errno;
    if(osaErrno == ENOENT)
      return osaErrno;
  }

  utimebuff.actime = statbuf.st_atime;
  utimebuff.modtime = statbuf.st_mtime;

  err = utime(to_file, &utimebuff);

  if(err == -1) {
    osaErrno = errno;
    return osaErrno;
  }

  return 0;
}

int osaFileExists(const char *filename)
{
  struct stat statbuf;
  int err;

  err = stat(filename, &statbuf);
  if(err == -1) {
    osaErrno = errno;
    if(osaErrno == ENOENT)
      return 0;
  }

  return 1;
}

time_t osaFileLastMod(const char *filename)
{
  struct stat statbuf;
  int err;

  err = stat(filename, &statbuf);
  if(!err)
    return statbuf.st_mtime;

  return -1;
}
