/* -*- C++ -*-
 ***********************************************************************
 *         libwsmake - Core functionality library of wsmake            *
 *        Copyright (C) 1999,2000,2001,2002 Michael Brownlow           *
 *                                                                     *
 * This program is free software; you can redistribute it and/or modify*
 * it under the terms of the GNU General Public License as published by*
 * the Free Software Foundation; either version 2 of the License, or   *
 * (at your option) any later version.                                 *
 *                                                                     *
 * This program is distributed in the hope that it will be useful,     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 * GNU General Public License for more details.                        *
 *                                                                     *
 * You should have received a copy of the GNU General Public License   *
 * along with this program; if not, write to the Free Software         *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
 *                                                                     *
 * For questions and comments, please email the author at:             *
 * mike@wsmake.org                                                     *
 ***********************************************************************/
#if !defined(__WSLOADER_H__)
#define __WSLOADER_H__ 1

#include <list>
#include <string>
#include <stdio.h>

namespace wsmake
{
  typedef enum {
    Site, ExpSeq, DataBlock, Part, Data,
    PrintExp, AssignExp, CommandExp, RegexExp, SubtagExp,
    PlusExp, MinusExp, DivideExp, TimesExp, UminusExp, UnotExp,
    PreIncExp, PostIncExp, PreDecExp, PostDecExp, LshiftExp, RshiftExp
  } ast_kind;

  typedef struct _ast {
    enum { i_ast, r_ast, v_ast, s_ast, n_ast } tag;
    union {
      long   i;  /* Let's not be morons and keep this simple, okay? */
      double r;  /* It's not terribly hard to figure out what these */
      char  *v;  /* one letter abbreviations stand for.             */
      char  *s;  /* Hint: v = variable; s = string                  */
      struct {
	ast_kind tag;
	struct _ast_list *l;
      } n;
    };
    const char *filename;
    long line;
  } ast;

  typedef struct _ast_list { 
    ast              *a;
    struct _ast_list *next;
  } ast_list;

  typedef struct _fsi {
    FILE *file;
    FILE *tfile;
    char *filename;
    long line;
  } fsi; /* file stack item */

  typedef enum { e_Scan, e_Parse } load_errors;

  ast_list *cons(ast *a, ast_list *l);
  ast *mk_node(const ast_kind tag, ast_list *v);
  ast *mk_int(const long l);
  ast *mk_real(const double r);
  ast *mk_var(const char *v);
  ast *mk_str(const char *s);
  void ast_dump(ast *a, int level = 0);
  void ast_free(ast *a);

  class wsLoader {
  public: 
    wsLoader(const list<string> *input_list);
    ~wsLoader();

    bool load(void);
    void newline(void);
    void newblock(void);
    void endblock(void);
    long currline(void);
    const char *currfile(void);
    long currblock(void);

    bool push(const char *filename);
    int wrap(void);

    void printError(int errtype, const char *topic, ...);

  private:
    fsi *pop(void);
    void splitArgs(char **ap, const char *arg_list);
    void memorizeFile(const char *file);
    const char *rememberFile(const char *file);

    long line;
    FILE *tfile;
    list<string> *input_list;
    string current_file;
    long *block_stack;
    long block_stack_depth;
    fsi **file_stack;
    long file_stack_depth;
    char **file_cache;
    long file_cache_length;
  };

  extern wsLoader *wsLoader_comp;
  extern char *ast_names[];
}
#else
namespace wsmake
{
  class wsLoader;
}
#endif /* __WSLOADER_H__ */
