#include <string>
#include <list>

#include <cstdio>
#include <cstdlib>

#include "config.h"
#include "osa.h"
#include "wsmake.h"

#if HAVE_GETOPT_LONG
# if LOCAL_GETOPT
#  include "../../getopt/getopt.h"
# else
#  include <getopt.h>
# endif
#else
# include <getopt.h>
#endif

using namespace wsmake;

void printOptions(void);
void printVersion(void);

int main(int argc, char **argv)
{
  wsLoader *loader;
  list<string> input_list;
  int opt;
  osaFileList_t *fl;
  char *ext;
  int exit_code = EXIT_SUCCESS;

  osaSetPrintLevel(3);

#if HAVE_GETOPT_LONG
  int option_index = 0;
  static struct option long_options[] = {
    {"silent",0,0,'s'},
    {"quiet",0,0,'s'},
    {"help",0,0,'h'},
    {"version",0,0,'v'},
    {"file",1,0,'f'},
    {"debug",1,0,'d'},
    {"wsmakefile",1,0,'f'}
  };
#endif /* HAVE_GETOPT_LONG */

#if HAVE_GETOPT_LONG
  while((opt=getopt_long(argc,argv,"shvf:d:",
                         long_options,&option_index)) != -1) {
#else
  while((opt=getopt(argc,argv,"shvf:d:")) != -1) {
#endif /* HAVE_GETOPT_LONG */
    switch(opt) {
    case 's':
    case 'q':
      osaSetPrintLevel(0);
      break;
    case 'h':
      printOptions();
      return EXIT_SUCCESS;
      break;
    case 'v':
      printVersion();
      return EXIT_SUCCESS;
      break;
    case 'f':
      input_list.push_back(optarg);
      break;
    case 'd':
      osaSetPrintLevel(atoi(optarg));
      break;
    case ':':
    case '?':
      exit(1);
      break;
    }
  }

  if(input_list.size() == 0) {
    if((fl = osaDirList(".")) != NULL) {
      while(fl != NULL) {
	if(!fl->directory) {
	  if(osaStrCmp(fl->filename, "wsmakefile") == 0) {
	    input_list.push_back(fl->filename);
	  } else {
	    if((ext=osaStrStr(fl->filename, ".ws")) != NULL) {
	      if(osaStrLen(ext) == 3) {
		input_list.push_back(fl->filename);
	      }
	    }
	  }
	}
	fl = fl->next;
      }
      fl = osaDirListFree(fl);
    }
  }

  loader = new wsLoader(&input_list);

  if(loader->load()) {
    delete loader;

    wsSite::typeCheck();
  } else {
    delete loader;
    exit_code = EXIT_FAILURE;
  }

  return exit_code;
}

void printOptions(void)
{
  osaPrintf(1, "Usage: %s [options] [target] ...\nOptions:\n", PACKAGE);
#if !HAVE_GETOPT_LONG
  osaPrintf(1, "  -f inputfile                     "
	    "Use file `inputfile' instead of standard input.\n");
  osaPrintf(1, "  -h                               "
	    "Print this message and exit.\n");
  osaPrintf(1, "  -s                               "
	    "No feedback.\n");
#else
  osaPrintf(1, "  -f inputfile, --file=inputfile   "
	    "Use file `inputfile' instead of standard input.\n");
  osaPrintf(1, "    --wsmakefile=inputfile\n");
  osaPrintf(1, "  -h,--help                        "
	    "Print this message and exit.\n");
  osaPrintf(1, "  -s, --silent, --quiet            "
	    "No feedback.\n");
#endif /* !HAVE_GETOPT_LONG */
  osaPrintf(1, "Report bugs to <bugs@wsmake.org>.\n");
}

void printVersion(void)
{
  osaPrintf(1, "%s %s (%s), Copyright (C) 1999-2002 Michael Brownlow\n\n",
	    PACKAGE, VERSION, __DATE__);
  osaPrintf(1, "Wsmake may be copied only under the terms of either the "
	    "Artistic License or\n");
  osaPrintf(1, "the GNU General Public License, which may be found in "
	    "the Wsmake %s source\n", VERSION);
  osaPrintf(1, "distribution.\n");
}
